/* ************************************************************************** */
// File       : pixelformat.cpp
// Programmer : Tobin Schwaiger-Hastanan
// Copyright (c) 1997-1999 Tobin Schwaiger-Hastanan
// Notes      : 
/* ************************************************************************** */

#include "pixelformat.h"

/*
required type definitions - 

typedef unsigned char uchar;
typedef unsigned long uint32;
*/

/* ************************************************************************** */
// CPixelFormat::CPixelFormat( ... )
// - CPixelFormat constructor
// - Initializes member variables
/* ************************************************************************** */
CPixelFormat::CPixelFormat( int BPP,		  uint32 RedMask,
							uint32 GreenMask, uint32 BlueMask,
							uint32 AlphaMask )
{
	m_BPP = BPP;

	m_RedMask	= RedMask;
	m_GreenMask = GreenMask;
	m_BlueMask  = BlueMask;
	m_AlphaMask = AlphaMask;
	
	m_RedShift   = GetShiftBits( m_RedMask );
	m_GreenShift = GetShiftBits( m_GreenMask );
	m_BlueShift  = GetShiftBits( m_BlueMask );
	m_AlphaShift = GetShiftBits( m_AlphaMask );

	m_RedBits   = GetBits( m_RedMask );
	m_GreenBits = GetBits( m_GreenMask );
	m_BlueBits  = GetBits( m_BlueMask );
	m_AlphaBits = GetBits( m_AlphaMask );
}

/* ************************************************************************** */
// CPixelFormat::BuildPixel( ... )
// - Builds a pixel from the floating point parameters provide to the pixel
//   format specified by the member variables.
// - Floating point color values range from 0.0 to 1.0
/* ************************************************************************** */
uint32 CPixelFormat::BuildPixel( float Red, float Green, float Blue,
				   float Alpha ) const
{
	uint32 pixel = 0;	
	
	pixel |= uint32( m_RedMask   * Red   ) & m_RedMask;
	pixel |= uint32( m_GreenMask * Green ) & m_GreenMask;
	pixel |= uint32( m_BlueMask  * Blue  ) & m_BlueMask;
	pixel |= uint32( m_AlphaMask * Alpha ) & m_AlphaMask;

	return pixel;
}

/* ************************************************************************** */
// CPixelFormat::BuildPixel( ... )
// - Builds a pixel from the uint32 parameters provide to the pixel
//   format specified by the member variables.
// - Integer color values range from 0 to 255
/* ************************************************************************** */
uint32 CPixelFormat::BuildPixel( uint32 Red, uint32 Green, uint32 Blue, 
				   uint32 Alpha ) const
{
	uint32 pixel = 0;

	Red   >>= 8 - m_RedBits;
	Green >>= 8 - m_GreenBits;
	Blue  >>= 8 - m_BlueBits;
	Alpha >>= 8 - m_AlphaBits;

	pixel |= ( Red   << m_RedShift   ) & m_RedMask;
	pixel |= ( Green << m_GreenShift ) & m_GreenMask;
	pixel |= ( Blue  << m_BlueShift  ) & m_BlueMask;
	pixel |= ( Alpha << m_AlphaShift ) & m_AlphaMask;

	return pixel;
}

/* ************************************************************************** */
// CPixelFormat::BuildPixel( ... )
// - Builds a pixel from the floating point parameters provide to the pixel
//   format specified by the member variables.
/* ************************************************************************** */
uint32 CPixelFormat::BuildPixel( const CColor& Color ) const
{
	uint32 pixel = 0;	
	
	pixel |= uint32( Color.GetRed()   * m_RedMask   ) & m_RedMask;
	pixel |= uint32( Color.GetGreen() * m_GreenMask ) & m_GreenMask;
	pixel |= uint32( Color.GetBlue()  * m_BlueMask  ) & m_BlueMask;
	pixel |= uint32( Color.GetAlpha() * m_AlphaMask ) & m_AlphaMask;

	return pixel;
}

/* ************************************************************************** */
// CPixelFormat::GetRedBits( ... )
// - Extract and returns the red bits from a pixel matching the pixel 
//   format specified by the member variables
// - Return value ranges from 0 to 255
/* ************************************************************************** */
uint32 CPixelFormat::GetRedBits( uint32 Pixel ) const
{
	Pixel   = ( Pixel & m_RedMask ) >> m_RedShift;
	Pixel <<= ( 8 - m_RedBits );
	return Pixel;
}

/* ************************************************************************** */
// CPixelFormat::GetGreenBits( ... )
// - Extract and returns the green bits from a pixel matching the pixel 
//   format specified by the member variables
// - Return value ranges from 0 to 255
/* ************************************************************************** */
uint32 CPixelFormat::GetGreenBits( uint32 Pixel ) const
{
	Pixel   = ( Pixel & m_GreenMask ) >> m_GreenShift;
	Pixel <<= ( 8 - m_GreenBits );
	return Pixel;
}

/* ************************************************************************** */
// CPixelFormat::GetBlueBits( ... )
// - Extract and returns the blue bits from a pixel matching the pixel 
//   format specified by the member variables
// - Return value ranges from 0 to 255
/* ************************************************************************** */
uint32 CPixelFormat::GetBlueBits( uint32 Pixel ) const
{
	Pixel   = ( Pixel & m_BlueMask ) >> m_BlueShift;
	Pixel <<= ( 8 - m_BlueBits );
	return Pixel;
}

/* ************************************************************************** */
// CPixelFormat::GetAlphaBits( ... )
// - Extract and returns the alpha bits from a pixel matching the pixel 
//   format specified by the member variables
// - Return value ranges from 0 to 255
/* ************************************************************************** */
uint32 CPixelFormat::GetAlphaBits( uint32 Pixel ) const
{
	Pixel   = ( Pixel & m_AlphaMask ) >> m_AlphaShift;
	Pixel <<= ( 8 - m_AlphaBits );
	return Pixel;
}

/* ************************************************************************** */
// CPixelFormat::ConvertStream( ... )
// - Converts a pixel stream of a specific pixel format to the pixel format
//   specified by the member variables.
// - Returns the number of bytes used in the destination buffer during 
//   conversion
/* ************************************************************************** */
uint32 CPixelFormat::ConvertStream( uchar* pDest, uint32 DestLength, const CPixelFormat& pfSource, uchar* pSrc, uint32 SrcLength ) const
{
	uchar* pDestEnd, *pSrcEnd;	
	int    SrcBytesPP, DestBytesPP;
	int    SrcBPP, DestBPP;
	uint32 SrcRed, SrcGreen, SrcBlue, SrcAlpha;

	SrcBPP  = pfSource.GetBPP();
	DestBPP = GetBPP();

	DestBytesPP = DestBPP >> 3;
	SrcBytesPP  = SrcBPP  >> 3;
	
	pDestEnd = pDest + DestLength;
	pSrcEnd  = pSrc  + SrcLength;

	while( pDest < pDestEnd && pSrc < pSrcEnd )
	{			
		uint32 SrcPixel = 0, DestPixel;
		
		for( int i = 0; i < SrcBytesPP; i ++)
			SrcPixel |= (uchar) (*pSrc++) << ( i * 8 );
		
		SrcRed    = pfSource.GetRedBits( SrcPixel );
		SrcGreen  = pfSource.GetGreenBits( SrcPixel );
		SrcBlue   = pfSource.GetBlueBits( SrcPixel );
		SrcAlpha  = pfSource.GetAlphaBits( SrcPixel );
		DestPixel = BuildPixel( SrcRed, SrcGreen, SrcBlue, SrcAlpha );

		for( i = 0; i < DestBytesPP; i ++ )		
			*pDest++ = (uchar) ( DestPixel >> (i * 8) );
	}
	
	return pDestEnd - pDest;
}